/*
* This is modified plugin to add fields to csv export from HS branch
* */
prism.run([

	'$locale',
	'base.services.$activity',
	'dashboard-base.services.$query',

	function ($locale, $activity, $query) {

		prism.fieldsToSwap = {};

		// this function is envoked from widget script
		// and sets fields that should be swapped during exporting csv file
		// in our case we should change widget filed with name and url to field with only name field, example shown below
		// prism.swapFields(widget, {fieldToBeChanged: "[Sheet1.NamePic]", fieldToBeUsed: "[Sheet1.name]"})
		prism.swapFields = function (widget, dims) {
			prism.fieldsToSwap[widget.oid] = dims;
		};

		prism.on("dashboardloaded", function (e, args) {
			if(args.dashboard.$$eventHandlers("widgetrender").indexOf(onWidgetRender)  >= 0) return; //check event handler exists

			args.dashboard.on("widgetrender",onWidgetRender);
		});

		function onWidgetRender(dashboard, args){
			//adding default function to Export to CSV
			if (!args.widget.exportToCSV) {
				args.widget.exportToCSV = function () {
					return exportToCSV(args.widget);
				};
			}
		}

		// export to csv command execute function
		var exportToCSV = function (widget) {
			// add indicator that widget is in export state.
			// this is needed when hooking to query events ("before/build/after) to analize the purpose of the query
			widget.isCsvExportInProgress = true;

			var query = createQuery(widget);

			downloadResult(widget, query);
			delete widget.isCsvExportInProgress; //reset state
		};

		//create query object and add new fields if necessary
		var createQuery = function(widget){
			var query = $query.buildWidgetQuery(widget);
			var swapFields = prism.fieldsToSwap[widget.oid];

			query.format = "csv";
			query.isMaskedResponse = true;
			query.filename = "widget.csv";
			query.download = true;
			query = $query.createJaqlRequestConfig(query);

			// change dim and column field for required fields
			if (swapFields) {
				query.metadata.forEach(function (metadataItem) {
					if (metadataItem.jaql && metadataItem.jaql.dim === swapFields.fieldToBeChanged) {
						metadataItem.jaql.dim = swapFields.fieldToBeUsed;
						// get column name from dimension name
						metadataItem.jaql.column = swapFields.fieldToBeUsed.split('.')[1].slice(0, -1);
					}
				});
			}

			// resetting count/offset
			query.count = 0;
			query.offset = 0;

			query.widget = widget.oid + ";" + widget.title;
			query.dashboard = widget.dashboard.oid + ";" + widget.dashboard.title;
			return query;
		};

		// send request and download csv file
		var downloadResult = function(widget, query){
			var datasource = query.datasource || widget.datasource;
			if (typeof datasource === "object") {
				datasource = datasource.title;
			}

			//query = encodeURIComponent(JSON.stringify(query));
			query = encodeURIComponent(angular.toJson(query));
			query = query.replace(/'/g, "%27");

			var $form = $('<form>', {
				"id": "exportToCSV",
				"html": "<input type='hidden' name='data' value='" + query + "'/>",
				"method": "post",
				"action": '/api/datasources/' + encodeURIComponent(datasource) + '/jaql/csv'
			}).appendTo(document.body).submit();

			$form.remove();

			// pushing activity
			$activity.pushWidget(widget, "exportwidgettocsv");
		};

		//gets items for CSV export
		var getCSVExportMenuItem = function (items, searchTerm) {
			var title = $$get(items[i],'command.title');

			for (var i = 0; i < items.length; i++) {
				if (defined(items[i], 'items')) {
					for (var j = 0; j < items[i].items.length; j++) {
						if (defined(items[i], 'items.' + j + '.command.execute') && $$get(items[i], 'items.' + j + '.command.title').indexOf(searchTerm) > -1) {
							return items[i].items[j];
						}
					}
				} else{
					if (defined(items[i].caption) && items[i].caption.indexOf(searchTerm) > -1) {
						return items[i];
					}

					if (defined(items[i], 'command.execute') && defined(title) && title.indexOf(searchTerm) > -1) {
						return items[i];
					}

				}
			}
		};

		//adds new CSV export function to widget (another filename)
		prism.on('beforemenu', function (event, args) {

			var searchTerm = 'CSV';
			var exportToCSVMenuItem = getCSVExportMenuItem(args.settings.items, searchTerm);

			if (exportToCSVMenuItem) {
				exportToCSVMenuItem.command.execute = function (e) {
					var widget = e ? e.widget : args.settings.scope.widget;
					return exportToCSV(widget);
				};
			}
		});
	}]);