prism.run([
	'$rootScope',
	'plugin-imagesInBarChart.services.bindOnce',

	function ($rootScope, $bindOnce) {

		let createdImages = [];

		prism.on('dashboardloaded', (se, args) => {
			let { dashboard } = args;
			let widgets = dashboard.widgets.$$widgets;

			// we need to handle addition of widgets, as user can duplicate current bar chart with images, and we need to
			// make required logic on it as well
			dashboard.on('widgetadded', (dashboard, options) => {
				let { widget } = options;
				initListeners(widget);
			});

			_.forEach(widgets, (widget) => { initListeners(widget); });

			function initListeners (widget) {
				// we need to find only bar charts widgets on dashboard, as the plugin is applied only to them
				if (widget.type === 'chart/bar') {
					$bindOnce(widget, 'beforeviewloaded', onBeforeViewLoaded);
					$bindOnce(widget, 'ready', onWidgetReady);

					widget.on('beforedatapointtooltip', (widget, context) => {
						context.context.category = context.context.category.split('---')[0];
					});
				}
			}

			function onBeforeViewLoaded (widget, args) {
				const appState = $('.db-preview-container').length > 0 ? 'preview' : $rootScope.appstate;
				// remove previously generated pictures
				if (createdImages[widget.oid]) {
					_.forEach(createdImages[widget.oid][appState], (image) => {
						$(image.element)[0].remove();
					});
				}

				createdImages[widget.oid] = {};
				createdImages[widget.oid][appState] = [];

				let options = args.options;
				options.plotOptions.bar = options.plotOptions.bar || {};
				options.plotOptions.bar.pointWidth = 1;

				let values = widget.metadata.panel("values").items;
				let breakByEnabled = widget.metadata.panel('break by').items.length !== 0;
				// make bar rectangular thin
				options.widgetOid = widget.oid;

				_.forEach(options.series, (seriesItem, index) => {
					//seriesItem.pointWidth = 1;

					// if widget value with corresponding index has useImage option - add image url to the
					// data item in current series. If break by is enabled - than add imageUrl to all series, but only for
					// that data items in seriesItem that has selectionData
					if (breakByEnabled || (values[index] && values[index].useImage)) {
						_.forEach(seriesItem.data, (dataItem) => {
							if (dataItem.selectionData) {
								dataItem.imageUrl = dataItem.selectionData[0].split('---')[1];
							}
						});
					}
				});

				// hide url part of label
				options.xAxis.labels.formatter = function () {
					return this.value.split('---')[0];
				};
			}

			function onWidgetReady (widget) {
				let element,
					chart;
				let appMode = $rootScope.appstate;

				const previewMode = $('.db-preview-container');

				// in preview mode we need to get chart from element's 'hc' data, as there is no corresponding chart in Highcharts.charts
				if (previewMode.length > 0) {
					appMode = 'preview';
					element = previewMode.find('.widget-body');
					chart = element.data('hc');
				} else {
					chart = _.find(Highcharts.charts, (chart) => {
						// sometimes in Highcharts.charts there are undefined values, so we need extra check on this
						if (chart && chart.options.widgetOid === widget.oid) {
							// Highchart.charts seems like never be cleared, thus it containes outdated links on chart objects
							// so wee need to check if specific chart object is in the DOM
							if ($(document).find(chart.container).length !== 0) {
								return true
							}
						}
						return false;
						// return chart ? chart.options.widgetOid === widget.oid : false;
					});
				}

				// need to check this due to phantom.js sometimes does not find chart object
				if (chart) {
					// hack for the break by functionality
					if (widget.metadata.panel('break by').items.length !== 0) {
						var imageUrlArray = _.pluck(chart.series[0].data, 'imageUrl');

						_.map(chart.series, (seriesItem, index) => {
							return index === 0 ? seriesItem : _.map(seriesItem.data, (dataItem, index) => {
								dataItem.imageUrl = imageUrlArray[index];
								return dataItem;
							});
						});
					}

					_.forEach(chart.series, (seriesItem, index, array) => {
						const imageSize = getImageSize(seriesItem.data.length, array.length);

						_.forEach(seriesItem.data, (point, index, array) => {
							if (point.imageUrl && point.plotY) {
								// we need slightly different logic for positive and negative values
								let delta = point.y >= 0 ? 0 : imageSize;

								// highchart renderer, see http://api.highcharts.com/highcharts/Renderer
								const image = chart.renderer.image(
									point.imageUrl,
									chart.containerWidth - chart.marginRight - point.plotY - delta, // horizontal alignment
									(chart.containerHeight - chart.marginBottom - point.plotX) - (imageSize / 2), // vertical alignment
									imageSize, imageSize // image width and height
								)
								.add();

								createdImages[widget.oid][appMode].push(image);
							}
						});
					});
				}
				}

			// we need to change image size depending on number of data on the chart, to prevent images overlapping
			function getImageSize(dataLength, seriesCount) {
				if (dataLength <= 5) {
					return seriesCount > 2 ? 70 : 50;
				} else if (dataLength <= 10) {
					return 40;
				} else if (dataLength <= 20) {
					return 30;
				} else {
					return 20;
				}
			}

		});
	}
]);